
// FPSBehaviour.java
// Andrew Davison, November 2005, ad@fivedots.coe.psu.ac.th

/* Every DELAY ms (1000ms; 1 sec), calculate the FPS and update the
   average FPS. 

   The FPS may drop to 0 if nothing is changing in the 3D scene.
   Moving the mouse will trigger frame generation.

   The wakeup condition uses WakeupOnElapsedTime(DELAY)
   rather than WakeupOnElapsedFrames(0) (as found in the Java 3D
   demo example, FPSCounter). WakeupOnElapsedTime will mean that
   the behaviour will be triggered less often, and so be less
   of an overhead than FPSCounter which will be executed in
   every frame.
*/

import java.util.*;
import java.text.DecimalFormat;

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.timer.J3DTimer;



public class FPSBehaviour extends Behavior
{
  private static final int DELAY = 1000;   // 1 second

  private WakeupCondition wakeUpCond;
  private long beforeTime, gameStartTime;
  private long prevFrameNo = 0;

  private double totalFPS = 0.0;
  private int count = 0;

  private DecimalFormat df;


  public FPSBehaviour()
  {
    wakeUpCond = new WakeupOnElapsedTime(DELAY);
    beforeTime = J3DTimer.getValue();
    gameStartTime = beforeTime;

    df = new DecimalFormat("0.#");  // 1 dp
  } // end of FPSBehaviour()


  public void initialize()
  {  wakeupOn(wakeUpCond);  }


  public void processStimulus(Enumeration criteria)
  {
    long timeNow = J3DTimer.getValue();
    long timeDiff = timeNow - beforeTime;
    double timeSpentInGame = ((double)(timeNow - gameStartTime)/1000000000L);  // ns --> secs

    if (timeDiff == 0) 
      System.out.println("FPS: 0 time difference");
    else {   // timeDiff > 0
      long frameNo = getView().getFrameNumber();
      long numFrames = frameNo - prevFrameNo;

      if (numFrames == 0)
        System.out.println("No frames");
      else {
        double fps = ((double)numFrames * 1000000000L)/timeDiff;
        totalFPS += fps;
        count++;

        System.out.println("Frame: " + frameNo + 
                         "; " + df.format(timeSpentInGame) + "s" +
                         "; FPS: " + df.format(fps) + 
                         "; AFPS: " + df.format(totalFPS/count) );

        prevFrameNo = frameNo;
      }
    }
    beforeTime = J3DTimer.getValue();
    wakeupOn(wakeUpCond);       // make sure we are notified again
  } // end of processStimulus()

} // end of FPSBehaviour class
